/*******************************************************************************
* Declarations                                                                 *
*******************************************************************************/
char *grog(/* name, nb, nw, codefile, tr, hz, layout, netlist, vhdl,
                    outline, datasheet */);
void grogdata(/* name, nb, nw, hz, tr, data */);
void groglayout(/* name, nb, nw, tr, hz, data, save */);
void grognetlist(/* name, nw, nb, hz */);
void grogoutline();
void grogpat(/* s, nb, nw, hz, data */);
void grogvhdl(/* s, nb, nw, hz, data */);
void grogcode(/* s, nw, nb, data */);
void grogicon(/* name, nb, nw, hz, r, msb */);
void randata();
char *long_bin();
char *long_hex();
char *biglong_bin();
char *biglong_hex();
long ln2p(/* n */);
/* nice :
   we should be able to read up to 64 bits.
   Tough for a 32 bits machine without cheating. */
typedef struct blong {long high, low;} biglong;

